#! /bin/sh
#+
#
# .. program:: mlintall.sh
#
# ===========
# mlintall.sh
# ===========
#
# -------------------------------------------
# mlintall.sh - run mlint for all existing .m
# -------------------------------------------
#
# SYNOPSIS
# ========
#
# ::
#
#   $ mlintall.sh -i indir -o outdir [-a mlint_option]
#
#
# DESCRIPTION
# ===========
#
# .. option:: -i <indir>
# .. option:: -o <outdir>
# .. option:: -a <mlint_option>
#
# ``mlintall.sh`` will launch :command:`mlint` command (MATLAB) for all
# existing .m files under ``indir``.
#
# If ``-a`` option is used, **mlint_option** will be added to :command:`mlint`
# command.
#
# Outputs of :command:`mlint` commands are in ``outdir``.
#
# Log file is written on :file:`{outdir}/mlintall.log.{YYYYMMDDTHHMMSSZ}`
#
# NOTES
# =====
#
# ``mlintrpt`` can also be used. Report will be visible via MATLAB browser.
#
# SEE ALSO
# ========
#
# http://blogs.mathworks.com/images/steve/92/handout_final_icip2006.pdf
#
# EXAMPLES
# ========
#
# To launch ``mlint`` on all m files under ${PROJECT}/src ::
#
#  $ mlintall.sh -i ${PROJECT}/src/ -o ${PROJECT_LOG}/
#
# To displays the McCabe complexity on all m files under ${PROJECT}/src ::
#
#  $ mlintall.sh -i ${PROJECT}/src/ -o ${PROJECT_LOG}/ -a cyc
#
# Results of ``mlint`` commands is written in some
# :file:`${PROJECT_LOG}/.\ *YYYYMMDDTHHMMSSZ*`
#
# TODO
# ====
#
# ++ option bavarde
#
# ++ machine dependant
#
# journal/sdtout matlab vs stdout unix
#
# evaluate relative directory
#
# error if outdir is in m_dir
#
# available license
#
# EVOLUTIONS
# ==========
#
# $URL$
#
# - fplod 20110304T134156Z zeus.locean-ipsl.upmc.fr (Linux)
#
#   * some unset
#
# - fplod 20110302T143106Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add -a option
#
# - fplod 20110225T111452Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * to avoid thousands of ``Warning: Type-ahead buffer overflow``
#     use -r run the command file, instead of input redirection
#     thanks to http://hpc.ucalgary.ca/quickstart/terminus/software/matlab
#
#   * evaluate directory .
#
# - fplod 2009-07-03T14:40:32Z zeus.locean-ipsl.upmc.fr (Linux)
#
#   * creation for TPOCTAVE project
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
      echo " www : no specific posix checking"
   ;;
   *)
      set -o posix
   ;;
esac
#
LANG=C
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage="Usage : ${command} -i indir -o outdir -a mlint_option"
#
mlint_option=""
#
minargcount=4
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
set +u
while [ ! -z "${1}" ]
do
   case "${1}" in
      -i)
         # directory for inputs choosen by user
         indir=${2}
         # si l'oprateur a mis ., on force  $(pwd) pour tre plus clair
         # dans les sorties
         if [ "${indir}" = "." ]
         then
            indir=$(pwd)
         fi
         shift
      ;;
      -o)
         # directory for outputs choosen by user
         outdir=${2}
         # si l'oprateur a mis ., on force  $(pwd) pour tre plus clair
         # dans les sorties
         if [ "${outdir}" = "." ]
         then
            outdir=$(pwd)
         fi
         shift
      ;;
      -a)
         mlint_option=${2}
         shift
      ;;
      *)
         # other choice
         echo "eee : unknown option ${1}"
         echo "${usage}"
         exit 1
      ;;
   esac
   # next flag
   shift
done
#
set -u
#
# check for ${indir}
if [ ! -d ${indir} ]
then
   echo " eee : ${indir} not found"
   exit 1
fi
#
# check for permission on indir
if [ ! -x ${indir} ]
then
   echo " eee : ${indir} not reachable"
   exit 1
fi
#
#++ readable
#
if [ ! -d ${outdir} ]
then
   mkdir -p ${outdir}
   echo "${command} : iii : creation of \${outdir}"
fi
# check for permission on outdir
if [ ! -x ${outdir} ]
then
   echo " eee : ${outdir} not reachable"
   exit 1
fi
if [ ! -w ${outdir} ]
then
   echo " eee : ${outdir} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "runtime=${log_date}" 1>> ${log}
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "indir=${indir}" 1>> ${log}
echo "outdir=${outdir}" 1>> ${log}
echo "mlint_option=${mlint_option}" 1>> ${log}

# test if matlab exist ++ and licence ok
#
# find .m files under ${indir}
list_m_files=$(find ${indir} -name "*.m" | sort)
#++ check 0 files
# find directories (to be added with addpath
list_m_dir=$(find ${indir} -name "*.m" -exec dirname {} \; | sort -u)
#
# buil add_path commands
rm ${outdir}/command_add_path.m 2> /dev/null
for m_dir in ${list_m_dir}
do
    echo "addpath('${m_dir}');" >> ${outdir}/command_add_path.m
done
unset m_dir
unset list_m_dir
#
memo_pwd=$(pwd)
# loop on each file to launch mlint on it
for m_file in ${list_m_files}
do
   # build the command
   arg_mlint=$(basename ${m_file} .m)
   if [ "${mlint_option}" = "" ]
   then
      command_matlab="mlint('${arg_mlint}');"
   else
      command_matlab="mlint('${arg_mlint}','-${mlint_option}');"
   fi
   cat ${outdir}/command_add_path.m >  ${outdir}/mlintall.m
   # write matlab command file
   {
      echo ${command_matlab}
      echo "exit"
   } >> ${outdir}/mlintall.m
   # change directory before launching matlab
   cd  ${outdir}/
   # launch matlab with this matlab command file (no extension)
   echo "${command} : iii : mlint of ${m_file}" >> ${log}
   echo "command : ${command_matlab}" >> ${log}
   matlab -nojvm -nosplash -nodisplay -r mlintall >> ${log}
   #++ debug echo "status fin matlab = $?"
   unset command_matlab
   unset arg_mlint
   rm mlintall.m
   cd ${memo_pwd}
done
unset list_m_files
unset m_file
unset memo_pwd
#
rm ${outdir}/command_add_path.m
#
unset usage
unset system
unset usage
unset log
unset log_date
#
#++set
# end
exit 0
