#! /bin/sh
#+
#
# .. program:: concat-files.sh
#
# .. _concat-files.sh:
#
# ===============
# concat-files.sh
# ===============
#
# SYNOPSIS
# ========
#
# ::
#
#  $ concat-files.sh -p product -b yyyymmdd -e yyyymmdd
#
# DESCRIPTION
# ===========
#
# .. option:: -p product name classif, pressure, height, temperature, tb087 ,tb097, tb108, tb108z, tb120, tb134
#
# .. option:: -b beginning date <yyyymmdd>
# .. option:: -e end date <yyyymmdd>
#
# Concatenate MSG files :file:`${PROJECT_ID}/MSG/{yyyy}/{mm}/{dd}/{yyyymmddhhmn}\_msg-{product}_map_15min.nc` between **yyyymmddb** and **yyyymmdde**
# in one file per day :file:`${PROJECT_OD}/MSG/{yyyy}/{mm}/msg-{produt}_{yyyy}-{mm}-{dd}_15min.nc`.
#
# Log file is written on :file:`${PROJECT_LOG}/concat-files.log.{YYYYMMDDTHHMMSSZ}`
#
# Output files can be used by :func:`plotimageMSG`, :func:`evolution_msg`, etc.
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph concat_files {
#           graph [
#           rankdir="TB",
#           ]
#           msg_t5 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/yyyymmddhhmn_msg-prod_map_15min.nc"];
#
#           msg_t6 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/msg-prod_yyyy-mm-dd_15min.nc"];
#
#           log [shape=ellipse,fontname=Courier,label="${PROJECT_LOG}/concat_files.log.{YYYYMMDDTHHMMSSZ}"];
#
#           concat_files [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/concat-files.sh",
#           label="${PROJECT}/src/concat-files.sh"];
#
#           {msg_t5} -> {concat_files} -> {log msg_t6}
#       } 
#
# EXAMPLES
# ========
#
# You already used :ref:`traite_msg-prod.sh` to create MSG NetCDF files 
# during [20060801, 20060802[ and you want to produce daily files::
#
#  $ concat-files.sh -p tb108 -b 20060801 -e 20060802
#
# And look at log file with ::
#
#  $ tlogd.sh concat-files
# 
# and of course on files in ${PROJECT_ID}.
#
# SEE ALSO
# ========
#
# :ref:`guide data MSG <data_msg>`
#
# Previous step : :ref:`traite_msg-prod.sh`
#
# Uses :
# NCO_
#
# .. _NCO: http://nco.sourceforge.net/
#
#
# TODO
# ====
#
# .. todo::
#
# add -f option
#
# build/complete catalog
#
# replace -files by _files (graphviz do not like concat-files and I rather like to have identidy between filename and
# dot entry of the software)
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# - pinsard 2011-06-01T09:18:41Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * correction on msg_t3 terminology (in fact msg_t5)
#
# - pinsard 2011-05-02T10:24:49Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * consolidation
#   * add parameter -p, -b and -e
#
# - pinsard 2010-07-23T08:38:46Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * suppress cd, input files are search in ${PROJECT_ID}/MSG/yyyy/mm/
#   
# - fplod 20100608T143812Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * minimal header
# - Karim Ramage ?
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
     echo "www : no specific posix checking"
     date_cmd=date
   ;;
   Darwin)
     set -o posix
     date_cmd=gdate
   ;;
   Linux)
     set -o posix
     date_cmd=date
   ;;
   *)
    set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-f] -p product -b yyyymmdd -e yyyymmdd"
#
hostname=$(hostname)
#
yyyymmddb_min=20060420
yyyymmdde_max=20061103
#
# default
debug=0
force=0
product=tb108
yyyymmddb=20060420
yyyymmdde=20060420
#
minargcount=6
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
     -p)
        product=${2}
        shift
     ;;
     -b)
        # first date to get
        yyyymmddb=${2}
        shift
     ;;
     -e)
        # last date to get
        yyyymmdde=${2}
        shift
     ;;
     -f)
        force=1
     ;;
     -h)
        echo "${usage}"
        exit 0
     ;;
     *)
        # other choice
        echo "eee : unknown option ${1}"
        echo "${usage}"
        exit 1
     ;;
   esac
   # next flag
   shift
done
unset usage
#
# check parameters
# and define associated variables
${date_cmd} -d "${yyyymmddb}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmddb}" < "${yyyymmddb_min}" ]]
then
    echo "${command} : eee : yyyymmddb ${yyyymmddb} must be equal or greater than yyyymmddb_min ${yyyymmddb_min}"
    exit 1
fi
#
status_date=${?}
if [ ${status_date} -ne 0 ]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} argument invalid"
    exit 1
fi
unset status_date
#
if [[ "${yyyymmdde}" > "${yyyymmdde_max}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be lower or equal to yyyymmdde_max ${yyyymmdde_max}"
    exit 1
fi
#
if [[ "${yyyymmdde}" < "${yyyymmddb}" ]]
then
    echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
    exit 1
fi
#
if [[ "${yyyymmdde}" = "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
case ${product} in
    classif)
    ;;
    pressure)
    ;;
    height)
    ;;
    temperature)
    ;;
    tb087)
    ;;
    tb097)
    ;;
    tb108)
    ;;
    tb108z)
    ;;
    tb120)
    ;;
    tb134)
    ;;
    *)
     echo "${command} : eee : product ${product} argument invalid"
     exit 1
    ;;
esac
#
tool=ncrcat
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
unset tool
#
tool=ncdump
type ${tool} 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
   echo "${command} : eee : tool ${tool} not found"
   exit 1
fi
unset status
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
   echo "${command} : eee : \${PROJECT_LOG} not defined"
   exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not found"
   exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not reachable"
   exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "product=${product}" 1>> ${log}
echo "yyyymmddb=${yyyymmddb}" 1>> ${log}
echo "yyyymmdde=${yyyymmdde}" 1>> ${log}
echo "" 1>> ${log}
#
# loop over all dates
current=$(${date_cmd} -u -d "${yyyymmddb}" +%s | awk '{printf "%10.10d",$1}')
end=$(${date_cmd} -u -d "${yyyymmdde}" +%s | awk '{printf "%10.10d",$1}')
delta=86400
while [[ "${current}" < "${end}" ]]
do
    echo "iii : looking for files of $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)" 1>> ${log}
    yyyymmdd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)
    yyyy=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y)
    mm=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%m)
    dd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%d)
    list=$(find ${PROJECT_ID}/MSG/${yyyy}/${mm}/${dd}/ -name "${yyyymmdd}????_msg-${product}_map_15min.nc" | sort)
    if [ "${list}" = "" ]
    then 
        echo "iii : no files found" 1>> ${log}
    else
        fileout=${PROJECT_OD}/MSG/${yyyy}/${mm}/msg-${product}_${yyyy}-${mm}-${dd}_15min.nc
        ncrcat -O ${list} ${fileout} >> ${log} 2>&1
        status=${?}
        if [ ${status} -ne 0 ]
        then
           echo "eee : pb with ncrcat $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)" 1>> ${log}
           for file in ${list}
           do
               ncdump -h ${file} >> ${log} 2>&1
           done
           exit 1
        else
           echo "iii : adding ${fileout}" >> ${log} 2>&1
        fi
    fi
    unset status
    unset yyyymmdd
    unset yyyy
    unset mm
    unset dd
    unset list
    # next time step
    current=$((${current} + ${delta}))
done
#
exit
