#! /bin/sh
#+
#
# .. program:: get_msg.sh
#
# .. _get_msg.sh:
#
# =====================================
# get_msg.sh -- get MSG reference files
# =====================================
#
# SYNOPSIS
# ========
#
# ::
#
#  $ get_msg.sh [-f] -p product -b yyyymmdd -e yyyymmdd
#
# DESCRIPTION
# ===========
#
# ``get_msg.sh`` get MSG reference files from **yyyymmddb** include to **yyyymmdde** excluded.
#
# .. option:: -f
# .. option:: -p product name classif, pressure, height, temperature, tb087 ,tb097, tb108, tb108z, tb120, tb134
# .. option:: -b beginning date <yyyymmdd>
# .. option:: -e end date <yyyymmdd>
#
# Log file is written on :file:`${PROJECT_LOG}/get_msg.log.{YYYYMMDDTHHMMSSZ}`
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph get_msg {
#           graph [
#             rankdir="TB",
#           ]
#
#           msg_ref_t1 [shape=diamond,fontname=Courier,label="loholt1.ipsl.polytechnique.fr:/bdd/AMMA/AOC-Paris/msg/yyyy/mm/dd/prod_msg01_yyyymmdd_hhmn_xxxxx.raw.gz"];
#           msg_ref_t2 [shape=diamond,fontname=Courier,label="loholt1.ipsl.polytechnique.fr:/bdd/AMMA/AOC-Paris/msg/yyyy/mm/dd/prod_msg01_yyyymmdd_hhmn_xxxxx.def.gz"];
#
#
#           msg_t1 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/prod_msg01_yyyymmdd_hhmn_xxxxx.raw.gz"];
#           msg_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_ID}/MSG/yyyy/mm/dd/prod_msg01_yyyymmdd_hhmn_xxxxx.def.gz"];
#
#           log [shape=ellipse,fontname=Courier,label="${PROJECT_LOG}/get_msg.log.YYYYMMDDTHHMMSSZ"];
#
#           get_msg [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/get_msg.sh",
#           label="${PROJECT}/src/get_msg.sh"];
#
#           {msg_ref_t1 msg_ref_t2} -> {get_msg} -> {log msg_t1 msg_t2}
#
#          }
#
# EXAMPLES
# ========
#
# You don't have any MSG reference data and you want tb108 during [20060801, 20060802[, you just have to run this tool ::
#
#  $ get_msg.sh -p tb108 -b 20060801 -e 20060802
#
# And look at log file with ::
#
#  $ tlogd.sh get_msg
#
# and of course on files in ${PROJECT_ID}.
#
# CAUTIONS
# ========
#
# how to know if there is enough place before ?
#
# SEE ALSO
# ========
#
# :ref:`data_msg`
#
# :ref:`varamma_profile.sh`
#
# Next step :
# :ref:`traite_msg-prod.sh`
#
# TODO
# ====
#
# .. todo::
#
# make it work outside climserv
#
# build/complete catalog 
#
# handle -f option
#
# describe next step traite_msg-prod.sh
#
# dirref_base /bdd/AMMA/AOC-Paris/msg/ avec def et raw ou /bdd/msg/images/xrit/2006 avec H_*
# cf discussion email avec KR initie le 20010427
#
# See strange behavour of date command ::
#
#  $ date -d '20060622T0645' +%Y%m%dT%H%M
#  20060622T0145
#  
# while 20060622T0645 expected !!
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# - pinsard 2011-05-02T13:39:47Z loholt1.ipsl.polytechnique.fr
#
#   * from **yyyymmddb** include to **yyyymmdde** excluded
#
# - pinsard 2011-04-29T14:09:33Z loholt1.ipsl.polytechnique.fr (Linux)
#
#   * improve time loop. tricky TZ pb !
#
# - fplod 20110428T072435Z 
#
#   * better handling of missing directories ans files
#   * make it work on loholt1 (climserv)
#   * thanks to http://tldp.org/LDP/abs/html/comparison-ops.html for strings comparison
#
# - fplod 20110427T141408Z cratos.locean-ipsl.upmc.fr (Linux)
#
#   * creation from traite_msg-prod.sh
#     thanks to http://stackoverflow.com/questions/1629015/bash-loop-between-two-given-dates 
#     for loop on dates
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
      echo "www : no specific posix checking"
      date_cmd=date
   ;;
   Darwin)
      set -o posix
      date_cmd=gdate
   ;;
   Linux)
      set -o posix
      date_cmd=date
   ;;
   *)
     set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-f] -p product -b yyyymmdd -e yyyymmdd"
#
hostname=$(hostname)
#
yyyymmddb_min=20060420
yyyymmdde_max=20081103
#
# default
force=0
product=tb108
yyyymmddb=20060801
yyyymmdde=20060802
#
minargcount=6
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
      -p)
         product=${2}
         shift
      ;;
      -b)
         # first date to get
         yyyymmddb=${2}
         shift
      ;;
      -e)
         # last date to get
         yyyymmdde=${2}
         shift
      ;;
      -f)
         force=1
      ;;
      -h)
         echo "${usage}"
         exit 0
      ;;
      *)
         # other choice
         echo "eee : unknown option ${1}"
         echo "${usage}"
         exit 1
      ;;
   esac
   # next flag
   shift
done
unset usage
#
# check parameters
# and define associated variables
#
case ${product} in
    classif)
        prefix="ClassifAMMAp2st"
    ;;
    pressure)
        prefix="ClassifAMMAp3sp"
    ;;
    height)
        prefix="ClassifAMMAp3sh"
    ;;
    temperature)
        prefix="ClassifAMMAp3st"
    ;;
    tb087)
        prefix="AtlSudMir087"
    ;;
    tb097)
        prefix="AtlSudMir097"
    ;;
    tb108)
        prefix="AtlSudMir108"
    ;;
    tb108z)
        prefix="AtlSudMir108"
    ;;
    tb120)
        prefix="AtlSudMir120"
    ;;
    tb134)
        prefix="AtlSudMir134"
    ;;
    *)
      echo "${command} : eee : product ${product} argument invalid"
      exit 1
    ;;
esac
#
${date_cmd} -u -d "${yyyymmddb}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ] 
then
     echo "${command} : eee : yyyymmddb ${yyyymmddb} argument invalid"
     exit 1
fi
unset status_date
#
if [[ "${yyyymmddb}" < "${yyyymmddb_min}" ]]
then
     echo "${command} : eee : yyyymmddb ${yyyymmddb} must be equal or greater than yyyymmddb_min ${yyyymmddb_min}"
     exit 1
fi
#
${date_cmd} -u -d "${yyyymmdde}" > /dev/null
status_date=${?}
if [ ${status_date} -ne 0 ] 
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} argument invalid"
     exit 1
fi
unset status_date
#
if [[ "${yyyymmdde}" > "${yyyymmdde_max}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be lower or equal to yyyymmdde_max ${yyyymmdde_max}"
     exit 1
fi
#
if [[ "${yyyymmdde}" < "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
if [[ "${yyyymmdde}" = "${yyyymmddb}" ]]
then
     echo "${command} : eee : yyyymmdde ${yyyymmdde} must be greater than yyyymmddb ${yyyymmddb}"
     exit 1
fi
#
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
   echo "${command} : eee : \${PROJECT_LOG} not defined"
   exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not found"
   exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not reachable"
   exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo "${command} : eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "hostname=${hostname}" 1>> ${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
unset log_date
echo "" 1>> ${log}
echo "[Parameters]" 1>> ${log}
echo "product=${product}" 1>> ${log}
echo "yyyymmddb=${yyyymmddb}" 1>> ${log}
echo "yyyymmdde=${yyyymmdde}" 1>> ${log}
echo "" 1>> ${log}
#
#
dirref_base=/bdd/AMMA/AOC-Paris/msg/
#
# loop over all dates to build URL lists
list_url=""
current=$(${date_cmd} -u -d "${yyyymmddb}" +%s | awk '{printf "%10.10d",$1}')
end=$(${date_cmd} -u -d "${yyyymmdde}" +%s | awk '{printf "%10.10d",$1}')
delta=900
while [[ "${current}" < "${end}" ]]
do
    echo "iii : looking for files of $(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%dT%H%M)" 1>> ${log}
    yyyymmdd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y%m%d)
    yyyy=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%Y)
    mm=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%m)
    dd=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%d)
    hhmn=$(${date_cmd} -u -d "1970-01-01 ${current} sec" +%H%M)
    dirref="${dirref_base}/${yyyy}/${mm}/${dd}"
    #
    # look for these reference files
    bfile_raw_gz=${prefix}_msg01_${yyyymmdd}_${hhmn}_?????.raw.gz
    file_raw_gz=${dirref}/${bfile_raw_gz}
    bfile_def_gz=${prefix}_msg01_${yyyymmdd}_${hhmn}_?????.def.gz
    file_def_gz=${dirref}/${bfile_def_gz}
    exist=0
    if [ ! -f ${file_raw_gz} ]
    then
        echo "iii : ${file_raw_gz} not found" >> ${log} 2>&1
    else
        echo "iii : ${file_raw_gz} found" >> ${log} 2>&1
        exist=$(( ${exist} + 1))
    fi
    if [ ! -f ${file_def_gz} ]
    then
        echo "iii : ${file_def_gz} not found" >> ${log} 2>&1
    else
        echo "iii : ${file_def_gz} found" >> ${log} 2>&1
        exist=$(( ${exist} + 1))
    fi
    if [ ${exist} -eq 0 ] 
    then
        echo "iii : missing both of ${file_raw_gz} or ${file_def_gz}" >> ${log} 2>&1 
    fi
    if [ ${exist} -eq 1 ]
    then
        echo "iii : missing one of ${file_raw_gz} or ${file_def_gz}" >> ${log} 2>&1 
    fi
    if [ ${exist} -eq 2 ] 
    then
        list_url="${list_url} ${file_def_gz} ${file_raw_gz}"
    fi
    unset yyyymmdd
    unset yyyy
    unset mm
    unset dd
    unset hhmn
    unset dirref
    #
    # next time step
    current=$((${current} + ${delta}))
done
unset current
unset end
#
if [ "${list_url}" = "" ]
then
   echo "iii : no files founds" >> ${log} 2>&1
else
    #
    # if file is not already in ${PROJECT_ID}, get it
    for url in ${list_url}
    do
        burl=$( basename ${url})
        after_msg1=${burl#*msg01_}
        url_yyyymmdd=${after_msg1%_hh}
        yyyy=${url_yyyymmdd:0:4}
        mm=${url_yyyymmdd:4:2}
        dd=${url_yyyymmdd:6:2}
        file=${PROJECT_ID}/MSG/${yyyy}/${mm}/${dd}/$(basename ${url})
        if [ ! -f ${file} ]
        then
           loc=$(dirname ${file})
           if [ ! -d ${loc} ]
           then
              mkdir -p ${loc}
              echo "iii : adding ${loc}" >> ${log}
           fi
           ln -s ${url} ${file}
           echo "iii : adding ${file}" >> ${log}
        else
           echo "iii : ${file} exists" >> ${log}
        fi
        unset yyyy
        unset mm
        unset dd
        unset burl
        unset after_msg1
        unset loc
    done
    unset file
    unset url
fi
unset list_url
#
unset command
unset log
unset hostname
unset usage
#
unset date_cmd
#
# end
#++set
exit 0
