;+
;
; ===================
; idl_amsu_netcdf.pro
; ===================
;
; .. function:: idl_amsu_netcdf(numch, temp, rtemp, rhdeg, datain, lats, lons, timesin)
;
; DESCRIPTION
; ===========
;
; A program for creating a netcdf file for AMSU data
;
; .. only:: man
;
;    Figure is visible on PDF and HTML documents only.
;
; .. only:: html or latex
;
;     .. graphviz::
;
;        digraph idl_amsu_netcdf {
;
;           datain [shape=tab, label="data 3d array lon, lat, time"]
;           lons [shape=tab, label="longitude 1d array"]
;           lats [shape=tab, label="latitude 1d array"]
;           timesin [shape=tab, label="time 1d array"]
;           temp [shape=tab, label="temp"]
;           rtemp [shape=tab, label="rtemp"]
;           rhdeg [shape=tab, label="rhdeg"]
;
;           amsu_t5 [shape=ellipse,
;               fontname=Courier,
;               label="${PROJECT_OD}/AMSU/numch_temprtrtemprhrhdeg_yyyymmdd_yyyymmdd_geomin_geomax.nc"];
;
;           idl_amsu_netcdf [shape=box,
;           fontname=Courier,
;           color=blue,
;           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/idl_amsu_netcdf.pro",
;           label="${PROJECT}/src/idl_amsu_netcdf.pro"];
;
;           {datain lons lats timesin maskin} -> {idl_amsu_netcdf} -> {amsu_t5}
;
;          }
;
; EXAMPLES
; ========
;
; To produce
; :file:`${PROJECT_OD}/AMSU/a5_20060801_20060812_01000w2000n_00100w3000n.nc`::
;
;    numch = 'a5'
;    temp = 'm'
;    rtemp = 0.5
;    rhdeg = 1.
;    lons = findgen(10) - 10
;    lats = findgen(11) + 20
;    timesin = indgen(12) + julday(8,1,2006L)
;    datain = findgen(size(lons,/dim), size(lats,/dim), size(timesin,/dim))
;    result = idl_amsu_netcdf(numch, temp, rtemp, rhdeg, datain, lats, lons, timesin)
;
; TIPS
; ====
;
; File produced by can be read using :func:`read_ncdf <saxo:read_ncdf>` ::
;
;  tb=read_ncdf('moyenne_tb', 20060801, 20060802, file='/homedata/pinsard/varamma_d/AMSU/a5mrt0.5rh1.0_20060801_20060804_02400w0400s_02400e1900n.nc')
;
; SEE ALSO
; ========
;
; :ref:`data_amsu`
;
; Called by :func:`cresamsu`
;
; use :func:`geolocation_to_string_idl`, :func:`jul2date <saxo:jul2date>`
;
; TODO
; ====
;
;
; ++ test si toutes les valeurs sont des valeurs manquantes/aberantes
; si ou ne rien crire
;
; improve tip
;
; check arg : time in julian days. check for noon !!
;
; unit of input vs attribute
;
; coding rules
;
; check time in netcdf
;
; convention CF, improve attributes
;
; realistic examples
;
; EVOLUTIONS
; ==========
;
; $Id$
;
; $URL$
;
; - fplod 20120430
;
;   * add temp, rtemp, rhdeg in example
;   * pro -> function
;   * time unlimited
;   * time origine is a ttribute of time not of tb
;   * fix double vs fltarr
;   * fix missing value
;
;   nb : now cdo showdate and showtime work
;
; - lelod 20120410
;
;   * ajout parametre temp pour choix des orbites interpolees  (m pour
;     morning, e pour evening)
;
; - fplod 20120410
;
;   * build output directory if needed
;   * remove prompt from examples
;
; - lelod 20111218
;
;   * la transformation du mask 1D en matrice est trop complexe et
;     inutilement lourde
;     proposition de recreer le masque avec le fichier bathy de S. masson (dans
;     cresamsu ou ici). Est ce utile du coup de l'inserer dans le fichier?
;
; - fplod 20111216T160934Z cratos (Linux)
;
;   * add a print to see filename on log
;   * fix landseamask unit
;
; - lelod 20111215
;
;   * add maskin in input parameters
;
; - fplod 20111130T111832Z cratos (Linux)
;
;   * replace use_amsua by numch in example
;   * chgt outputfile terminology in graph
;   * fix time pb : using ncview 1.93 we can see now "frame 1/12 1-Aug-2006"
;
; - fplod 20111108T131624Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * complete header
;   * new parameters
;   * attribute name : nom blank allowed
;   * add compile_opt
;
; - lelod 20111108
;
;   * reprise
;
; - C. Lesage, 03/08/2005
;
;   * creation
;
;-
function idl_amsu_netcdf, numch, temp, rtemp, rhdeg, datain, lats, lons, timesin
;
compile_opt idl2, strictarrsubs
;
@cm_project
;
s = size(SCOPE_TRACEBACK(/STRUCTURE),/DIMENSION)
routine = (SCOPE_TRACEBACK(/STRUCTURE))[s-1].Routine
;
key_performance=0
if key_performance EQ 1 THEN BEGIN
    time1 = SYSTIME(1)
ENDIF
;
; Return to caller if errors
ON_ERROR, 2
;
result = -1
;
usage = 'result = idl_amsu_netcdf(numch, temp, rtemp, rhdeg, datain, lats, lons, timesin)'
nparam = N_PARAMS()
IF (nparam NE 8) THEN BEGIN
    ras = report(['Incorrect number of arguments.' $
          + '!C' $
          + 'Usage : ' + usage])
    return, result
ENDIF
;
nblat = size(lats,/dim)
nblon = size(lons,/dim)
nbtime = size(timesin,/dim)
;
; build output filename
; avec le nom et le numro du canal trait par interpolation de
; Cresmann
rt=string(rtemp,format='(f3.1)')
rh=string(rhdeg,format='(f3.1)')
prefix = numch + temp + 'rt' + rt + 'rh' + rh + '_'
;
look = 'filename'
scale = 100.
geomin = geolocation_to_string_idl(min(lons), min(lats), look, scale)
geomax = geolocation_to_string_idl(max(lons), max(lats), look, scale)
timemin = min(timesin)
yyyymmddb = jul2date(timemin)
timemax = max(timesin)
yyyymmdde = jul2date(timemax)
cdfout = project_id_env+ 'AMSU/' $
        + prefix+string(yyyymmddb,format='(I8.8)') + '_' $
        + string(yyyymmdde,format='(I8.8)') + '_' $
        + geomin + '_' $
        + geomax $
        + '.nc'
;
; definition des tableaux qui vont contenir les valeurs  crire
; dans les fichiers NetCDF
moyenne_tb = fltarr(nblon, nblat, nbtime)
xlon = fltarr(nblon)
ylat = fltarr(nblat)
jours = fltarr(nbtime)
;
moyenne_tb[*,*,*] = datain[*,*,*]
xlon[*] = lons
ylat[*] = lats
jours[*] = timesin - julday(01,01,2000)
;
; masque des valeurs aberrantes
ind = where(finite(moyenne_tb,/nan), nnbad)
if nnbad ne 0 then begin
   moyenne_tb[ind]=-9999.
endif
msg = report(['iii : ' + routine + ' : nb valeurs manquantes = ' + string(nnbad)])
;
; ++ test si toutes les valeurs sont des valeurs manquantes/aberantes
; si ou ne rien crire
;
;
;---------------------------------------
; creation du fichier netcdf de sortie
; --------------------------------------
;
; buid the directory if needed
dirout = file_dirname(cdfout,/MARK_DIRECTORY)
IF (FILE_TEST(dirout, /DIRECTORY, /EXECUTABLE, /WRITE) EQ 0) THEN BEGIN
    FILE_MKDIR, dirout
ENDIF
;
msg = report(['iii : ' + routine + ' : ouverture pour ecriture de ' + cdfout])
id = ncdf_create(cdfout,/clobber)
ncdf_control, id, /nofill
;
; dimension
dimidx = ncdf_dimdef(id, 'xlon', nblon)
dimidy = ncdf_dimdef(id, 'ylat', nblat)
dimidt = ncdf_dimdef(id, 'time', /UNLIMITED)
;
; attributs globaux
ncdf_attput, id, 'File_Name', cdfout, /GLOBAL
ncdf_attput, id, 'IDL_Program_Name', 'idl_amsu_netcdf', /GLOBAL
;print,'creation date'
creation_date = string(jul2date(SYSTIME(/JULIAN, /UTC )), format='(I8)')
ncdf_attput, id, 'Creation_date', creation_date, /GLOBAL
;print, creation_date
;
; declaration des variables
varid = lonarr(4)
varid[0] = ncdf_vardef(id, 'xlon', [dimidx], /float)
varid[1] = ncdf_vardef(id, 'ylat', [dimidy], /float)
varid[2] = ncdf_vardef(id, 'time', [dimidt], /float)
varid[3] = ncdf_vardef(id, 'moyenne_tb',[dimidx,dimidy,dimidt], /float)
;
; attributs de chaque variable
ncdf_attput, id, varid[0], 'units', 'degrees_east'
ncdf_attput, id, varid[0], 'title', 'Longitude'
;
ncdf_attput, id, varid[1], 'units', 'degrees_north'
ncdf_attput, id, varid[1], 'title', 'Latitude'
;
ncdf_attput, id, varid[2], 'units', 'days since 2000-01-01 00:00:00'
ncdf_attput, id, varid[2], 'long_name', 'time'
ncdf_attput, id, varid[2], 'short_name', 'time'
ncdf_attput, id, varid[2], 'axis', 'T'
ncdf_attput, id, varid[2], 'time_origin', '2000-01-01 00:00:00'
;
ncdf_attput, id, varid[3], 'short_name', 'amsu_tb'
ncdf_attput, id, varid[3], 'missing_value', -9999.00
ncdf_attput, id, varid[3], 'unit', 'degree_kelvin'
;
; remplissage de l'entete
ncdf_control, id, /endef
ncdf_varput, id, varid[0], xlon
ncdf_varput, id, varid[1], ylat
ncdf_varput, id, varid[2], jours
;
; remplissage des variables
ncdf_varput, id, varid[3], moyenne_tb
;
; fermeture du fichier
ncdf_close, id
;
result = 0
return, result
;
end
