#! /bin/sh
#+
#
# .. _olr_split.sh:
#
# .. program:: olr_split.sh
#
# ============
# olr_split.sh
# ============
#
# -------------------------
# split OLR mean daily file
# -------------------------
#
# SYNOPSIS
# ========
#
# ``olr_split.sh [-g] [-r] [-f] [-y year]``
#
# DESCRIPTION
# ===========
#
# .. option:: -g
# .. option:: -r
# .. option:: -f
# .. option:: -y <year>
#
# Split OLR mean daily file by year and reduce geographical domain
# to [60  W, 50  E] [30  S, 45  N].
#
# If :option:`-g` option is set, there will be no geographical domain reduction.
#
# If :option:`-f` option is set, previously output files will be overwritten.
#
# If :option:`-y` option is set with a year, only this year will be splitted.
#
# For validation of the tool, if :option:`-r` option is set, output files will be
# concatenated in one (might be usefull, combinated with :option:`-g` option to
# check).
#
# ``olr_split.sh`` read :file:`${PROJECT_ID}/olr.day.mean.nc` (reference data).
#
# If :option:`-g` option is set, ``olr_split.sh`` write
# :file:`${PROJECT_OD}/olr.day.mean_{yyyy}.nc`.
#
# If :option:`-g` option isn't set ``olr_split.sh`` write
# :file:`${PROJECT_OD}/olr.day.mean_{yyyy}_ng.nc`.
#
# These output files can be plot using :ref:`hovmuller.pro`.
#
# Log file is written on :file:`${PROJECT_LOG}/olr_split.log.{YYYYMMDDTHHMMSSZ}`
#
# .. only:: man
#
#    Figure is visible on PDF and HTML documents only.
#
# .. only:: html or latex
#
#     .. graphviz::
#
#        digraph olr_split {
#           graph [
#           rankdir="TB",
#           ]
#
#           olr_t1 [shape=diamond,fontname=Courier,label="${PROJECT_ID}/olr.day.mean.nc"];
#
#           olr_t2 [shape=ellipse,fontname=Courier,label="${PROJECT_OD}/olr.day.mean_yyyy.nc"];
#           olr_t3 [shape=ellipse,fontname=Courier,label="${PROJECT_OD}/olr.day.mean_yyyy_ng.nc"];
#
#           olr_split [shape=box,
#           fontname=Courier,
#           color=blue,
#           URL="http://forge.ipsl.jussieu.fr/varamma/browser/trunk/src/olr_split.sh",
#           label="${PROJECT}/src/olr_split.sh"];
#
#           {olr_t1} -> {olr_split} -> {olr_t2};
#           {olr_t1} -> {olr_split} -> {olr_t3};
#
#          }
#
# EXAMPLES
# ========
#
# To split :file:`${PROJECT_ID}/olr.day.mean.nc` between [2000,2007] and between
# [60  W, 50  E] [30  S, 45  N]::
#
#  $ olr_split.sh
#
# To see log file::
#
#  $ tlogd.sh olr_split
#
#
# To split :file:`${PROJECT_ID}/olr.day.mean.nc` in 2006 between
# [60  W, 50  E] [30  S, 45  N]::
#
#  $ olr_split.sh -y 2006
#
# To split globaly and build a recomposed file::
#
#  $ olr_split.sh -g -r
#
# One might compare original and rebuild file::
#
#  $ cdo diffv ${PROJECT_ID}/olr.day.mean.nc ${PROJECT_OD}/olr.day.mean_rebuild.nc
#
# diff should only show differences in global attributes
#
# CAUTIONS
# ========
#
# Does'nt work on zeus because of UDUnits library problems
#
# ++++ GROS DOUTE SUR LES LONGITUDES DANS CES FICHIERS +++++++
#
# TODO
# ====
#
# .. todo::
#
#
# ++ option debug/verbose
#
# ++ vrification accs filein
#
# SEE ALSO
# ========
#
# :ref:`varamma_profile.sh`
#
# :ref:`cat_clim_olr.sh`
#
# :ref:`hovmuller.pro`
#
# EVOLUTIONS
# ==========
#
# $Id$
#
# - fplod 20100629T123019Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * explicit limit of graph visibility
#
# - fplod 20100608T154000Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add graph (from varamma/notes/varamma_etape2.dot)
#
# - fplod 20100318T075630Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   *  unset
#
# - fplod 2009-04-08T15:51:13Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add details on input and output files
#
# - pinsard 2009-04-01T10:40:18Z camelot.ipsl.polytechnique.fr
#
#   * force overriding on NCO commands to avoid interactive behaviour
#
#   * force LANG=POSIX to avoid side effect of >LANG=fr_FR.UTF-8
#     in commands like echo "11" | awk ' {printf "%+4.1f",$1}' : +11,0
#     instead of 11.0
#
#   * write log in logfile
#
# - fplod 2008-08-14T14:53:04Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * modify West longitude conversion according to
#     http://en.wikipedia.org/wiki/Longitude
#
# - fplod 2008-08-13T14:26:41Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * add -f option
#   * add -y option
#   * add 180 degre to longitude centered on meridien 0 (<0 =W , >0=E)
#
# - fplod 2008-07-24T15:02:12Z aedon.locean-ipsl.upmc.fr (Darwin)
#
#   * creation
#
#-
system=$(uname)
case "${system}" in
   AIX|IRIX64)
      echo " www : no specific posix checking"
   ;;
   *)
     set -o posix
   ;;
esac
unset system
#
set -u
#
LANG=POSIX
#
case $(hostname) in
   zeus.locean-ipsl.upmc.fr|dedale.locean-ipsl.upmc.fr)
      echo "eee : pb on UDUnits library on zeus"
      echo "eee : contact system team"
      exit 1
   ;;
esac
#
command=$(basename ${0})
log_date=$(date -u +"%Y%m%dT%H%M%SZ")
#
usage=" Usage : ${command} [-r] [-g] [-f]"
#
# ++ test si ncks available
#
# default
force=0
latmin=-30
latmax=45
lonmin=$((-60 + 360))
lonmax=50
global=0
yearmin=2000
yearmax=2007
rebuild=0
#
minargcount=0
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
unset minargcount
#
while [ ${#} -gt 0 ]
do
   case ${1} in
      -g)
         global=1
         # ++ comment connaitre la 1er anne si -g.
         # dans le README on nous dit que a commence en 1974 mais il faudrait automatiser
         # ++ idem pour la dernire anne
         # pour l'instant forage
         yearmin=1974
         yearmax=2008
         echo "www : forage yearmin yearmax : ${yearmin} ${yearmax}"
      ;;
      -f)
         force=1
      ;;
      -r)
         rebuild=1
      ;;
      -y)
         yearmin=${2}
         yearmax=${yearmin}
         shift
      ;;
      -h)
         echo "${usage}"
         exit 0
      ;;
      *)
         # other choice
         echo "eee : unknown option ${1}"
         echo "${usage}"
         exit 1
      ;;
   esac
   # next flag
   shift
done
unset usage
#
# ++ check validity of yearmin and yearmax
#
# check for ${PROJECT_LOG} definition
if [ "${PROJECT_LOG}" = "" ]
then
   echo " eee : \${PROJECT_LOG} not defined"
   exit 1
fi
#
# check for ${PROJECT_LOG} existence
if [ ! -d ${PROJECT_LOG} ]
then
   echo " eee : ${PROJECT_LOG} not found"
   exit 1
fi
#
# check for permission access on PROJECT_LOG
if [ ! -x ${PROJECT_LOG} ]
then
   echo " eee : ${PROJECT_LOG} not reachable"
   exit 1
fi
#
# check for write permission on PROJECT_LOG
if [ ! -w ${PROJECT_LOG} ]
then
   echo " eee : ${PROJECT_LOG} not writable"
   exit 1
fi
#
log=${PROJECT_LOG}/$(basename ${0} .sh).log.${log_date}
echo "[Context]" 1>> ${log}
echo "command=$(basename ${0})" 1>>${log}
echo "runtime=${log_date}" 1>> ${log}
echo "log=${log}" 1>> ${log}
#
# check for input file
filein=${PROJECT_ID}/olr.day.mean.nc
if [ ! -r ${PROJECT_ID}/olr.day.mean.nc ]
then
   echo "eee : ${filein} not accessible" 1>> ${log}
   exit 1
fi
#
# build date string for ncks command
list_split=""
iyear=${yearmin}
while [ ${iyear} -le ${yearmax} ]
do
   timemin=$(echo ${iyear} | awk ' {printf "%4.4d-01-01 00:00:00.0",$1}')
   timemax=$(echo ${iyear} | awk ' {printf "%4.4d-12-31 00:00:00.0",$1}') #++23:59:59
   echo "iii : extraction ${iyear} from ${filein}" 1>> ${log}
   if [ ${global} -eq 0 ]
   then
      fileout=${PROJECT_OD}/$(basename ${filein} .nc)_${iyear}_ng.nc
   else
      fileout=${PROJECT_OD}/$(basename ${filein} .nc)_${iyear}.nc
   fi
   if [ -f ${fileout} ]
   then
      if [ ${force} -eq 0 ]
      then
         echo "eee : ${fileout} already exists" 1>> ${log}
         exit 1
      else
         # I remove the file because I don't know how to tell ncks to ask interactivily if I want to append, overwrite etc...
         rm -f ${fileout}
      fi
   fi
   if [ ${global} -eq 0 ]
   then
      # build float representation of lat and long limit (for hyperslab)
      clatmin=$(echo ${latmin} | awk ' {printf "%+3.1f",$1}')
      clatmax=$(echo ${latmax} | awk ' {printf "%+3.1f",$1}')
      clonmin=$(echo ${lonmin} | awk ' {printf "%+4.1f",$1}')
      clonmax=$(echo ${lonmax} | awk ' {printf "%+4.1f",$1}')
      ncks \
         -O \
         --dimension time,"${timemin}","${timemax}" \
         --dimension lat,"${clatmin}","${clatmax}" \
         --dimension lon,"${clonmin}","${clonmax}" \
         ${filein} ${fileout}
      status_ncks=${?}
      if [ ${status_ncks} -ne 0 ]
      then
         echo "eee : pb with ncks ${filein}" 1>> ${log}
         exit 1
      else
         list_split="${list_split} ${fileout}"
      fi
      unset status_ncks
   else
      ncks \
         -O \
         --dimension time,"${timemin}","${timemax}" \
         ${filein} ${fileout}
      status_ncks=${?}
      if [ ${status_ncks} -ne 0 ]
      then
         echo "eee : pb with ncks ${filein}" 1>> ${log}
         exit 1
      else
         list_split="${list_split} ${fileout}"
      fi
      unset status_ncks
   fi
   iyear=$(( ${iyear} + 1 ))
done
unset iyear
#
echo "iii : list of splitted files : ${list_split}" 1>> ${log}
#
if [ ${rebuild} -eq 1 ]
then
   # if rebuild is asked, concatenation of all the splitted files
   filetot=${PROJECT_OD}/$(basename ${filein} .nc)_rebuild.nc
   rm -f  ${filetot} 2> /dev/null
   ncrcat ${list_split} ${filetot}
   status_ncrcat=${?}
   if [ ${status_ncrcat} -ne 0 ]
   then
      echo "eee : pb with ncrcat" 1>> ${log}
      exit 1
   else
      echo "iii : rebuild result in ${filetot}" 1>> ${log}
   fi
   unset status_ncrcat
fi
#
# end
unset log
exit 0
