function [result]=plot_zi(lonmin, lonmax, latmin, latmax, west_value, east_value, north_value, south_value, zi_code, zi_description)

%PLOT_ZI plot zones of interest on a world map

%+
%
% =========
% plot_zi.m
% =========
%
% .. function:: plot_zi(lonmin, lonmax, latmin, latmax, west_value, east_value, north_value, south_value, zi_code, zi_description)
%
% DESCRIPTION
% ===========
%
% Plot zones of interest on a world map delimited by **lonmin**, **lonmax**, **latmin**,
% **latmax**.
%
% Figure is saved in :file:`./plot_precip.ps`
%
%    :param lonmin: longitude min of the plot, W < 0
%    :units lonmin: deg
%    :type lonmin: double
%    :raise lonmin: required
%
%    :param lonmax: longitude max of the plot, W < 0
%    :units lonmax: deg
%    :type lonmax: double
%    :raise lonmax: required
%
%    :param latmin: latitude min of the plot, N > 0
%    :units latmin: deg
%    :type latmin: double
%    :raise latmin: required
%
%    :param latmax: latitude max of the plot, N > 0
%    :units latmax: deg
%    :type latmax: double
%    :raise latmax: required
%
%    :param west_value: west most longitudes
%    :type west_value: 1d array of doubles
%    :raise west_value: required
%
%    :param east_value: east most longitudes
%    :type east_value: 1d array of doubles
%    :raise east_value: required
%
%    :param north_value: north most latitudes
%    :type north_value: 1d array of doubles
%    :raise north_value: required
%
%    :param south_value: south most latitudes
%    :type south_value: 1d array of doubles
%    :raise south_value: required
%
%    :param zi_code: codes of the zone of interest
%    :type zi_code: 1D cell array of strings
%    :raise zi_code: required
%
%    :param zi_description: descriptions of the zone of interest
%    :type zi_description: 1D cell array of strings
%    :raise zi_description: required
%
%    :returns: 0 if ok or -1 if error
%    :rtype: integer
%
% EXAMPLES
% ========
%
% From scratch to plot some simulated zones of interest:
%
% see demo 1
%
% To plot precipitation from :file:`${PROJECT_ID}/maps/varamma_zi.csv`:
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`zones`
%
% :ref:`varamma_startup.m`
%
% :func:`read_zi`
% :func:`simul_zi`
%
% TODO
% ====
%
% check arg
%
% ref to traitdecode
%
% better plot
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/plot_zi.m $
%
% - fplod 20110920T132345Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * difference or syntax in octave 3.2.4 vs 3.4.0
%
% - fplod 20110704T090622Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add x and y labels
%   * improve test "polygon inside"
%
% - fplod 20110630T155327Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add plot of the rectangle of each zi
%
% - fplod 20110629T092743Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
global application;
global application_version;
%
result=-1;
%
usage='[result]=plot_zi(lonmin, lonmax, latmin, latmax, west_value, east_value, north_value, south_value, zi_code, zi_description)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('lonmin');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lonmin ' arg_info.class]);
   whos lonmin
   error(usage);
end
%
arg_info=whos('lonmax');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lonmax ' arg_info.class]);
   whos lonmax
   error(usage);
end
%
arg_info=whos('latmin');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg latmin ' arg_info.class]);
   whos latmin
   error(usage);
end
%
arg_info=whos('latmax');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg latmax ' arg_info.class]);
   whos lonmax
   error(usage);
end
%
%
arg_info=whos('west_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg west_value' arg_info.class]);
   whos west_value
   error(usage);
end
clear arg_info
%
if ~isvector(west_value)
   disp('arg west_value must be a vector');
   error(usage);
end
%
arg_info=whos('east_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg east_value ' arg_info.class]);
   whos east_value
   error(usage);
end
clear arg_info
%
if ~isvector(east_value)
   disp('arg east_value must be a vector');
   error(usage);
end
%
arg_info=whos('north_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg north_value' arg_info.class]);
   whos north_value
   error(usage);
end
clear arg_info
%
if ~isvector(north_value)
   disp('arg north_value must be a vector');
   error(usage);
end
%
arg_info=whos('south_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg south_value ' arg_info.class]);
   whos south_value
   error(usage);
end
clear arg_info
%
if ~isvector(south_value)
   disp('arg south_value must be a vector');
   error(usage);
end
%
clear arg_info
%
%
arg_info=whos('zi_code');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg zi_code ' arg_info.class]);
   whos zi_code
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg zi_code must be a cell array of strings');
%
%
arg_info=whos('zi_description');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg zi_description ' arg_info.class]);
   whos zi_description
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg zi_description must be a cell array of strings');
%
switch application
  case {'matlab'}
     test_consistent=isequal(numel(west_value), numel(east_value), numel(north_value), numel(south_value), numel(zi_code), numel(zi_description));
  case {'octave'}
     test_consistent=size_equal(west_value, east_value, north_value, south_value, zi_code, zi_description);
  otherwise
     error('unknown application running');
end
%
if (not(test_consistent))
   disp(['eee : inconsistent dimensions of west_value, east_value, north_value, south_value, zi_code et zi_description']);
   whos west_value
   whos east_value
   whos north_value
   whos south_value
   whos zi_code
   whos zi_description
   error(usage);
end
%
% define axis not to close from area to be plot
ymin=latmin - 2;
ymax=latmax + 2;
xmin=lonmin - 2;
xmax=lonmax + 2;
%
% read world map
load('cmgco_traitdecote.mat','Xgco','Ygco');
%
% initialiaze a new figure
ifigure=gcf()+1;
figure(ifigure);
%
% plot world map (black dots)
plot(Xgco,Ygco,'k.','MarkerSize',1)
clear Xgco
clear Ygco
% limit world map around latmin,latmax and lonmin, lonmax
axis([lonmin lonmax latmin latmax]);
%
% define an area (polygon) to check if zone of interest is inside it
x_area_region=[lonmin lonmax lonmax lonmin lonmin];
y_area_region=[latmin latmin latmax latmax latmin];
% for each zone of interest, write name on the map at lat and lon position
for index_zi=1:size(zi_description,1)
  % check if geolocation of the zone of interest has NaN
  switch application
      case {'matlab'}
         some_nan=(isnan(west_value(index_zi)) || isnan(east_value(index_zi)) || isnan(north_value(index_zi)) || isnan(south_value(index_zi)));
      case {'octave'}
          switch application_version
              case {'3.4.0'}
                  some_nan=or(isnan(west_value(index_zi)),isnan(east_value(index_zi)),isnan(north_value(index_zi)),isnan(south_value(index_zi)));
              case {'3.2.4'}
                  some_nan=(isnan(west_value(index_zi)) || isnan(east_value(index_zi)) || isnan(north_value(index_zi)) || isnan(south_value(index_zi)));
              otherwise
                  error(['unknown application version running ', application , application_version '\n']);
          end
      otherwise
         error('unknown application running');
   end

  if some_nan
     warning([ 'zone of interest #' num2str(index_zi) ' NaN geolocation']);
     disp([ 'west_value: ' num2str(west_value(index_zi))]);
     disp([ 'east_value : ' num2str(east_value(index_zi))]);
     disp([ 'north_value: ' num2str(north_value(index_zi))]);
     disp([ 'south_value : ' num2str(south_value(index_zi))]);
  else
     % check if zone of interest is inside area
     ws_in_area = inpolygon(west_value(index_zi), south_value(index_zi),x_area_region,y_area_region);
     wn_in_area = inpolygon(west_value(index_zi), north_value(index_zi),x_area_region,y_area_region);
     en_in_area = inpolygon(east_value(index_zi), north_value(index_zi),x_area_region,y_area_region);
     es_in_area = inpolygon(east_value(index_zi), south_value(index_zi),x_area_region,y_area_region);
     switch application
        case {'matlab'}
           some_corners_out=(~ws_in_area || ~wn_in_area || ~en_in_area || ~es_in_area);
        case {'octave'}
           some_corners_out=or(~ws_in_area,~wn_in_area,~en_in_area,~es_in_area);
        otherwise
            error('unknown application running');
     end

     if some_corners_out
        warning([ 'zone of interest #' num2str(index_zi) ' outside of the box']);
        disp([ 'west_value: ' num2str(west_value(index_zi))]);
        disp([ 'east_value : ' num2str(east_value(index_zi))]);
        disp([ 'north_value: ' num2str(north_value(index_zi))]);
        disp([ 'south_value : ' num2str(south_value(index_zi))]);
        disp([ 'box: ' '[', num2str(lonmin) ',' num2str(latmin) ']' '[' num2str(lonmax) ',' num2str(latmax) ']' ]);
    else
        disp([ 'iii : zone of interest #' num2str(index_zi) ' inside of the box']);
        disp([ 'west_value: ' num2str(west_value(index_zi))]);
        disp([ 'east_value : ' num2str(east_value(index_zi))]);
        disp([ 'north_value: ' num2str(north_value(index_zi))]);
        disp([ 'south_value : ' num2str(south_value(index_zi))]);
        disp([ 'box: ' '[', num2str(lonmin) ',' num2str(latmin) ']' '[' num2str(lonmax) ',' num2str(latmax) ']' ]);
        label=strcat(zi_description(index_zi,:), 'ws');
        text(west_value(index_zi), south_value(index_zi),label);
        label=strcat(zi_description(index_zi,:), 'wn');
        text(west_value(index_zi), north_value(index_zi),label);
        label=strcat(zi_description(index_zi,:), 'en');
        text(east_value(index_zi), north_value(index_zi),label);
        label=strcat(zi_description(index_zi,:), 'es');
        text(east_value(index_zi), south_value(index_zi),label);
        % define the rectangle and plot it
        x=zeros(5);
        x=NaN;
        y=zeros(5);
        y=NaN;
        index_polygon=1;
        x(index_polygon)=east_value(index_zi);
        y(index_polygon)=south_value(index_zi);
        index_polygon=index_polygon+1;
        x(index_polygon)=east_value(index_zi);
        y(index_polygon)=north_value(index_zi);
        index_polygon=index_polygon+1;
        x(index_polygon)=west_value(index_zi);
        y(index_polygon)=north_value(index_zi);
        index_polygon=index_polygon+1;
        x(index_polygon)=west_value(index_zi);
        y(index_polygon)=south_value(index_zi);
        index_polygon=index_polygon+1;
        x(index_polygon)=east_value(index_zi);
        y(index_polygon)=south_value(index_zi);
        hold on
        plot(x,y,'-');
     end
     clear in_area
     clear label
  end
end
xlabel('Longitude');
ylabel('Latitude');
%
hold on
%++plot(x_area_region,y_area_region,'-')
%
% limit world map around latmin,latmax and lonmin, lonmax + 2 on each side
axis([xmin xmax ymin ymax]);
clear xmin;
clear xmax;
clear ymin;
clear ymax;
%
clear lonmin;
clear lonmax;
clear latmin;
clear latmax;
clear x_area_region;
clear y_area_region;
%
% save figure
printer='ps';
print_printer=['-d', printer];
fullfilename=['./', mfilename, '.', printer];
clear printer;
print(print_printer,fullfilename);
clear print_printer;
clear fullfilename;
%
result=0;
%
end

%!demo
%!
%! % From scratch to plot some simulated zones of interest::
%!
%! clear all
%! close all
%! varamma_startup
%! more off
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! index_simulation = uint8(1);
%! [west_value, east_value, north_value, south_value, zi_code, zi_description] = simul_zi(index_simulation);
%! result = plot_zi(lonmin, lonmax, latmin, latmax, west_value, east_value, north_value, south_value, zi_code, zi_description);

%!demo
%!
%! % To plot precipitation from :file:`${PROJECT_ID}/maps/varamma_zi.csv`::
%!
%! clear all
%! close all
%! varamma_startup
%! more off
%! lonmin = -20;
%! lonmax = -10;
%! latmin = 10;
%! latmax = 20;
%! [west_value, east_value, north_value, south_value, zi_code, zi_description]=read_zi();
%! result = plot_zi(lonmin, lonmax, latmin, latmax, west_value, east_value, north_value, south_value, zi_code, zi_description);
