function [datestr_value, precip_value]=read_precip(station, yyyy, datasettype)

%READ_PRECIP fill array precipitation 2D array and associated array datestr
% with data in file :file:`${PROJECT_ID}/PRECIP/{station}_pluie_{yyyy}.csv`
% or file :file:`${PROJECT_ID}/PRECIP/pluvio_{station}_{yyyy}.csv`

%
%+
%
% =============
% read_precip.m
% =============
%
% .. function:: read_precip(station, yyyy, datasettype)
%
% DESCRIPTION
% ===========
%
%    :param yyyy: year
%    :type yyyy: uint16
%    :raise yyyy: required
%
%    :param station: station
%    :type station: string
%    :raise station: required
%
%    :param datasettype: datasettype precip_cm or precip_cd
%    :type datasettype: string
%    :raise datasettype: required
%
% read precipitation data in a
% file :file:`${PROJECT_ID}/PRECIP/{station}_pluie_{yyyy}.csv`
% if datasettype is set to ``precip_cm`` (like column month) or
% in a file :file:`${PROJECT_ID}/PRECIP/pluvio_{station}_{yyyy}.csv`
% if datasettype is set to ``precip_cd`` (like column day).
%
% Les prcipitations sont stockes dans le tableau 2D (geolocation,date) **precip_value**
%
% Les dates au format `yyyymmdd` sont stockes dans le tableau 1D **datestr_value**.
%
% EXAMPLES
% ========
%
% To read :file:`${PROJECT_ID}/PRECIP/barkedji_pluie_2006.csv`:
%
% see demo 1
%
% A file :file:`${PROJECT_ID}/PRECIP/barkedji_pluie_2006_write_precip.csv` can be written
% with data we just read::
%
%  octave> result=write_precip(station, datestr_value, precip_value, datasettype);
%
% Files comparison might help detecting problems (either on files or in programs involved)::
%
%  $ diff ${PROJECT_ID}/PRECIP/barkedji_pluie_2006.csv ${PROJECT_ID}/PRECIP/barkedji_pluie_2006_write_precip.csv
%
%
% To read :file:`${PROJECT_ID}/PRECIP/pluvio_linguere_2006.csv`:
%
% see demo 2
%
% A file :file:`${PROJECT_ID}/PRECIP/pluvio_linguere_2006_write_precip.csv` can be written
% with data we just read::
%
%  octave> result=write_precip(station, datestr_value, precip_value,datasettype);
%
% Files comparison might help detecting problems (either on files or in programs involved)::
%
%  $ diff ${PROJECT_ID}/PRECIP/pluvio_linguere_2006.csv ${PROJECT_ID}/PRECIP/pluvio_linguere_2006_write_precip.csv
%
% SEE ALSO
% ========
%
% :ref:`guide data precipitation <data_precipitation_insitu>`
%
% :func:`simul_precip`
% :func:`write_precip`
% :func:`plot_precip`
%
% TODO
% ====
%
% status de retour
%
% description du rsultat en ReST
%
% pourquoi 20061201 : parce que -99.9 dans la case 20061131 qui n'existe pas
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/read_precip.m $
%
% - fplod 20110816T073113Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * replace int16 by uint16 for integer parameters
%   * replace int8 by uint8 for integer parameters
%
% - fplod 20110228T133126Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * use int2str instead of num2str
%
% - fplod 20110224T101530Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * remove code in examples (because are already in demo
%   * add comments in demo
%
% - fplod 20110223T151034Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * add demo and test
%
% - fplod 20110222T135551Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * avoid :samp:`Warning: NaN converted to int8(0)`
%   * nicer case for findind present months
%
% - fplod 20110221T154746Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * remove simulated geolocation
%
% - fplod 20110124T105932Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * introducing parameter datasettype precip_cm or precip_cd
%   * add reading :file:`pluvio_{station}_{yyyy}.csv`
%
% - fplod 20110114T155147Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * header line should be written using digit month
%   * remove extra precip_value NaN values due to preallocation
%
% - fplod 20110114T102014Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * reorganize init and file column/row to date index conversion
%
% - fplod 20110112T102527Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
%
global PROJECT_ID
%
usage='[datestr_value, precip_value]=read_precip(station, yyyy, datasettype)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('yyyy');
if ~strcmp(arg_info.class,'uint16')
   disp(['Incorrect type of arg yyyy ' arg_info.class]);
   whos yyyy
   error(usage);
end
clear arg_info
%
arg_info=whos('station');
if ~strcmp(arg_info.class,'char')
   disp(['Incorrect type of arg station ' arg_info.class]);
   whos station
   error(usage);
end
clear arg_info
%
arg_info=whos('datasettype');
if ~strcmp(arg_info.class,'char')
   disp(['Incorrect type of arg datasettype ' arg_info.class]);
   whos station
   error(usage);
end
clear arg_info
%
%
% build preciptitation filename to be read
switch datasettype
   case {'precip_cm'}
      fullfilename=[PROJECT_ID 'PRECIP/' station '_pluie_' int2str(yyyy) '.csv'];
   case {'precip_cd'}
      fullfilename=[PROJECT_ID 'PRECIP/' 'pluvio_' station '_' int2str(yyyy) '.csv'];
   otherwise
      error(['eee : unknown datasettype ' datasettype])
end
disp(['iii : opening for reading ', fullfilename]);
% opening file
[fid,message]=fopen(fullfilename,'r');
% test if fullfilename exists
if (fid == -1)
    disp(message)
    error([ fullfilename ' not exist'])
end
%
% the maximum of day is the number of days in a month
days_in_month_max = 31;
% the maximum of month is the number of months in a year
months_in_year_max = 12;
% the maximum of geolocation is 1 in this type of file
nb_geolocation_max = 1;
%
% preallocation of datestr_value array to missing value 9999999
nb_datestr_max=months_in_year_max*days_in_month_max;
datestr_value=repmat('99999999',nb_datestr_max,nb_geolocation_max);
%
% preallocation of precip_value array to missing value NaN
precip_value=zeros(nb_geolocation_max,days_in_month_max*months_in_year_max);
precip_value(:,:)=NaN;
%
% read file
%
switch datasettype
   case {'precip_cm'}
      % first line contains "Jours," and an unknown quantity of months
      % so we read this line and loop to find the number of columns
      %
      % get next line as a string
      header_line = fgetl(fid);
      %
      % loop to find the number of columns
      index_column = 1;
      mm=zeros(months_in_year_max,1,'uint8');
      while ( ~isempty(header_line) )
        % parse next column label
        [next,header_line] = strtok(header_line,',');
        switch next
           case {'06','07','08','09','10','11'}
              mm(index_column)=str2double(next);
           case {'"Days"'}
           otherwise
              error(['eee : unknown column ' next])
        end
        clear next
        % increase index of column
        index_column = index_column + 1;
      end
      % close file
      fclose(fid);
      %
      % we know now how many columns are written on header
      clear index_column
      %
      % remove extra mm NaN values due to preallocation
      %++mm=mm(~isnan(mm));
      %
      % reading data from file (from second line till end of file)
      data = dlmread(fullfilename,',', 1, 0);
      %
      % put values into arrays to be used for plot or computation
      dd_value=data(:,1);
      %
      % note : precip_rd_cm for precip array arrange with column = month and row = day
      precip_rd_cm=data(:,2:size(data,2));
      %
      % test if dd_value is identical to 1:1:days_in_month_max
      dd_value_ref=ones(1,days_in_month_max,'uint8');
      dd_value_ref=transpose(1:1:days_in_month_max);
      dd_ok=isequal(dd_value_ref,dd_value);
      if (~dd_ok)
         whos dd_value_ref
         whos dd_value
         dd_value_ref
         dd_value
         error('eee : dd_value and dd_value_ref not identical');
      end
      clear dd_value_ref
      %
      clear nb_dd
      %
      % fill precip_value and datestr_value
      nb_precip_column= size(precip_rd_cm,2);
      nb_precip_row= size(precip_rd_cm,1);
      index_date=1;
      index_geolocation=1;
      % loop on column
      for precip_icolumn=1:nb_precip_column
          % loop on row
          for precip_irow=1:nb_precip_row
              % test if precip is NaN or negative
              precip_nan=isnan(precip_rd_cm(precip_irow,precip_icolumn));
              precip_neg=(precip_rd_cm(precip_irow,precip_icolumn) < 0.);
              precip_mv=or(precip_nan,precip_neg);
              if (~precip_mv)
                 % compute date of current column/row
                 index_mm = precip_icolumn + 1;
                 dmonth=double(mm(index_mm));
                 clear index_mm
                 index_dd = precip_irow;
                 dday=double(dd_value(index_dd));
                 clear index_dd
                 dyyyy=double(yyyy);
                 datestr_value(index_date,:)=datestr(datenum(dyyyy,dmonth,dday),'yyyymmdd');
                 precip_value(index_geolocation,index_date,index_geolocation)=precip_rd_cm(precip_irow,precip_icolumn);
                 % debug disp(['precip_irow = ' num2str(precip_irow) ' precip_icolumn = ' num2str(precip_icolumn) ' date = ' datestr_value(index_date,:) ' precip_rd_cm = ' num2str(precip_rd_cm(precip_irow,precip_icolumn)) ]);
                 index_date = index_date + 1;
              end
              clear precip_nan
              clear precip_neg
              clear precip_mv
          end
      end
   case {'precip_cd'}
      % reading data from file (from first to last
      data = dlmread(fullfilename,',', 0, 0);
      %
      % put values into arrays to be used for plot or computation
      yyyy_value=data(:,1);
      % ++ test if year in file is equal to argument
      clear yyyy_value
      mm=data(:,2);
      %
      % test if months in file between 1 and 12, not dupe, etc.
      precip_rm_cd=data(:,3:size(data,2));
      %
      % fill precip_value and datestr_value
      nb_precip_column= size(precip_rm_cd,2);
      nb_precip_row= size(precip_rm_cd,1);
      index_date=1;
      index_geolocation=1;
      % loop on row
      for precip_irow=1:nb_precip_row
          % loop on column
          for precip_icolumn=1:nb_precip_column
              % test if precip is NaN or negative
              precip_nan=isnan(precip_rm_cd(precip_irow,precip_icolumn));
              precip_neg=(precip_rm_cd(precip_irow,precip_icolumn) < 0.);
              precip_mv=or(precip_nan,precip_neg);
              if (~precip_mv)
                 % compute date of current column/row
                 index_mm = precip_irow;
                 dmonth=double(mm(index_mm));
                 clear index_mm
                 dday=double(precip_icolumn);
                 dyyyy=double(yyyy);
                 datestr_value(index_date,:)=datestr(datenum(dyyyy,dmonth,dday),'yyyymmdd');
                 precip_value(index_geolocation,index_date,index_geolocation)=precip_rm_cd(precip_irow,precip_icolumn);
                 % debug disp(['precip_irow = ' num2str(precip_irow) ' precip_icolumn = ' num2str(precip_icolumn) ' date = ' datestr_value(index_date,:) ' precip_rm_cd = ' num2str(precip_rm_cd(precip_irow,precip_icolumn)) ]);
                 index_date = index_date + 1;
              end
              clear precip_nan
              clear precip_neg
              clear precip_mv
          end
      end
      clear precip_rm_cd
   otherwise
      error(['eee : unknown datasettype ' datasettype])
end
% remove extra precip_value NaN values due to preallocation
precip_value=precip_value(~isnan(precip_value));
% resize extra datestr_value due to preallocation
datestr_value=datestr_value(1:size(precip_value,2),:);
clear mm
clear nb_precip_column
clear nb_precip_row
clear precip_irow
clear precip_icolumn
clear data
%
% ++
%whos
%
end

%!demo
%! % To read :file:`${PROJECT_ID}/PRECIP/barkedji_pluie_2006.csv`::
%! varamma_startup
%! more off
%! yyyy = uint16(2006);
%! station = 'barkedji';
%! datasettype = 'precip_cm';
%! [datestr_value, precip_value] = read_precip(station, yyyy, datasettype);

%!demo
%! % To read :file:`${PROJECT_ID}/PRECIP/pluvio_linguere_2006.csv`::
%! varamma_startup
%! more off
%! yyyy = uint16(2006);
%! station = 'linguere';
%! datasettype = 'precip_cd';
%! [datestr_value, precip_value] = read_precip(station, yyyy, datasettype);

%!test
%! varamma_startup
%! more off
%! yyyy = uint16(2006);
%! station = 'barkedji';
%! datasettype = 'precip_cm';
%! [datestr_value, precip_value] = read_precip(station, yyyy, datasettype);

%!test
%! varamma_startup
%! more off
%! yyyy = uint16(2006);
%! station = 'linguere';
%! datasettype = 'precip_cd';
%! [datestr_value, precip_value] = read_precip(station, yyyy, datasettype);
