function [result]=write_lai_gis(lon_value, lat_value, datejul_value, datestr_value, lai_value, classes_value, msdnom_value, zi_code, yyyy)

%WRITE_LAI_GIS write LAI 2D array and associated arrays date, lat, lon, vegetation classe, pedologiy classe, into
%file :file:`${PROJECT_ID}/LAI_CSV/ :file:`${PROJECT_ID}/LAI/LAI{yyyy}F_veg2010_sol_{zi_code}_write_lai_gis.csv`
%from associated arrays lat, lon, etc.

%
%+
%
% ===============
% write_lai_gis.m
% ===============
%
% .. function:: write_lai_gis(lon_value, lat_value, datejul_value, datestr_value, lai_value, classes_value, msdnom_value, zi_code, yyyy)
%
% DESCRIPTION
% ===========
%
%    :param lon_value: longitude array
%    :type lon_value: 1D array of floats
%    :raise lon_value: required
%
%    :param lat_value: latitude array
%    :type lat_value: 1D array of floats
%    :raise lat_value: required
%
%    :param datestr_value: date (yyyymmdd) array
%    :type datestr_value: 1D array of strings
%    :raise datestr_value: required
%
%    :param lai_value: LAI array
%    :type lai_value: 2D array of floats
%    :raise lai_value: required
%
%    :param zi_code: zi_code ex 'barkedji'
%    :type zi_code: string
%
%    :param yyyy: yyyy to put written in filename
%    :type yyyy: unit16
%
%    ++
%
%    :returns: 0 if ok or -1 if error
%    :rtype: integer
%
%
% Write in a file
% file :file:`${PROJECT_ID}/LAI/LAI{yyyy}F_veg2010_sol_{zi_code}_write_lai_gis.csv`
% LAI 2D array and associated arrays date, lat, lon, vegetation classe, pedologiy classe.
%
% EXAMPLES
% ========
%
% From scratch to write some simulated LAI GIS data in
% :file:`${PROJECT_ID}/LAI/LAI2008F_veg2010_sol_simul_write_lai_gis.csv`
%
% see demo 1
%
% From real file :file:`${PROJECT_ID}/LAI/LAI2008F_veg2010_sol_barkedji.csv`
% to write  LAI GIS data in
% :file:`${PROJECT_ID}/LAI/LAI2008F_veg2010_sol_simul_write_lai_gis.csv`
%
% see demo 2
%
% SEE ALSO
% ========
%
% :ref:`data_lai`
% :ref:`zones`
%
% :func:`simul_lai_gis` ++ :func:`simul_lai_2d`
% :func:`read_lai_gis`
%
% TODO
% ====
%
% yyyy parameter may be computed according to datestr_values values
%
% arg checking
%
% common code with write_lai_2d.m : externalize !!
%
% usage of simul_lai_gis when written, now simul_lai_2d used and NaN and silly for GIS data
%
% EVOLUTIONS
% ==========
%
% $Id$
%
% $URL: svn+ssh://pinsard@forge.ipsl.jussieu.fr/ipsl/forge/projets/varamma/svn/trunk/src/write_lai_gis.m $
%
%
% - fplod 20110920T091450Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * replace num2str(yyyy) by int2str(yyyy) because octave error
%     on cratos
%
% - fplod 20110916T073154Z cratos (Linux)
%
%   * suppress unused data
%   * add parameter zi_code
%   * add parameter yyyy
%   * fix format
%
% - fplod 20110816T084401Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * handle IO error
%
% - fplod 20110708T150518Z aedon.locean-ipsl.upmc.fr (Darwin)
%
%   * creation
%
%-
%
global application;
global application_version;
%
global PROJECT_ID;
%
result=-1;
%
usage='result=write_lai_gis(lon_value, lat_value, datejul_value, datestr_value, lai_value, classes_value, msdnom_value, zi_code, yyyy)';
%
if nargin~=nargin(mfilename)
   disp(['Incorrect number of input arguments ' num2str(nargin)]);
   error(usage);
end
%
if nargout ~= nargout(mfilename)
   disp(['Incorrect number of output arguments ' num2str(nargout)]);
   error(usage);
end
%
arg_info=whos('lon_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lon_value' arg_info.class]);
   whos lon_value
   error(usage);
end
clear arg_info
%
if ~isvector(lon_value)
   disp('arg lon_value must be a vector');
   error(usage);
end
%
arg_info=whos('lat_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lat_value ' arg_info.class]);
   whos lat_value
   error(usage);
end
clear arg_info
%
if ~isvector(lat_value)
   disp('arg lat_value must be a vector');
   error(usage);
end
%
arg_info=whos('datejul_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg datejul_value' arg_info.class]);
   whos datejul_value
   error(usage);
end
clear arg_info
%
if ~isvector(datejul_value)
   disp('arg datejul_value must be a vector');
   error(usage);
end
%
arg_info=whos('lai_value');
if ~strcmp(arg_info.class,'double')
   disp(['Incorrect type of arg lai_value ' arg_info.class]);
   whos lai_value
   error(usage);
end
clear arg_info
%
if ~ismatrix(lai_value)
   disp('arg lai_value must be a vector');
   %++error(usage);
end
%
clear arg_info
%
arg_info=whos('classes_value');
if ~strcmp(arg_info.class,'cell')
   disp(['Incorrect type of arg classes_value ' arg_info.class]);
   whos classes_value
   error(usage);
end
clear arg_info
%
% +todo+   disp('arg classes_value must be a cell array of strings');
%
arg_info=whos('zi_code');
if ~strcmp(arg_info.class,'char')
    disp(['Incorrect type of arg zi_code' arg_info.class]);
    whos zi_code
    error(usage);
end
clear arg_info
%
arg_info=whos('yyyy');
if ~strcmp(arg_info.class,'uint16')
    disp(['Incorrect type of arg yyyy ' arg_info.class]);
    error(usage);
end
clear arg_info
%
% check for size of arg ++
nb_line_data=size(lai_value,2);
nb_datestr=size(datestr_value,1);
%
% calculate doy
datenum_value=zeros(nb_datestr,1);
datenum_value(:)=NaN;
for index_datestr=1:nb_datestr
   datenum_value(index_datestr)=datenum(datestr_value(index_datestr,:),'yyyymmdd');
end
clear index_datestr
yyyy_floatvalue=str2double(datestr(datenum_value(1),'yyyy'));
doy_value=datenum_value(:) - datenum(yyyy_floatvalue,1,1) + 1;
clear datenum_value
%
% build ASCII filename to be written
fullfilename=[ PROJECT_ID 'LAI_CSV/LAI' int2str(yyyy) 'F_veg2010_sol_' zi_code '_' mfilename '.csv'];
disp(['iii : opening for writing ', fullfilename]);
[fid,message]=fopen(fullfilename,'w');
if (fid == -1)
    disp(message)
    error([ fullfilename ' can not be written'])
end
clear fullfilename
%
% write header with days of year
format_header=['"X";"Y"' repmat(';%3.3d',1,nb_datestr) ';"classes";"MSDNOM"\n'];
fprintf(fid, format_header,transpose(doy_value(:)));
clear format_header
clear doy_value
%
% write data
%
sep=';';
% convert cell to string if needed
switch application
  case {'matlab'}
     classes_value = char(classes_value);
     msdnom_value = char(mapusub_value);
  case {'octave'}
     switch application_version
        case {'3.2.3','3.2.4','3.4.0'}
           classes_value = char(classes_value);
           msdnom_value = char(msdnom_value);
        otherwise
           error(['unknown application version running ', application , ' ', application_version '\n']);
     end
  otherwise
     error('unknown application running');
end
format_msdnom='%s';
format_classes='%s';
format_lat='%-6.3f';
format_lon='%-7.3f';
format_lai='%5.1f';
% loop on lines to write each data line
for index_line_data=1:nb_line_data
    msdnom_trim=strtrim(sprintf(format_msdnom,msdnom_value(index_line_data,:)));
    classes_trim=strtrim(sprintf(format_classes,classes_value(index_line_data,:)));
    lon_trim=strtrim(sprintf(format_lon,lon_value(index_line_data)));
    lat_trim=strtrim(sprintf(format_lat,lat_value(index_line_data)));
    lai_string = '';
    for index_datestr=1:nb_datestr
	lai_trim = strtrim(sprintf(format_lai,lai_value(index_datestr,index_line_data)));
	lai_string = [lai_string lai_trim ';'];
    end
    fprintf(fid, '%s%s%s%s%s%s%s%s\n' ...
                       , lat_trim(:) ...
                       , sep ...
                       , lon_trim(:) ...
                       , sep ...
		       , lai_string(:) ...
                       , classes_trim(:) ...
                       , sep ...
                       , msdnom_trim(:));
end
clear format_data
clear sep
%
fclose(fid);
clear fid
%
%++whos;
result=0;
%
end

%!demo
%! % From scratch to write some simulated LAI GIS data in
%! % :file:`${PROJECT_ID}/LAI_CSV/LAI2008F_veg2010_sol_simul_write_lai_gis.csv`
%! varamma_startup
%! more off
%! index_simulation = uint8(1);
%! [lon_value, lat_value, datejul_value, datestr_value, lai_value] = simul_lai_2d(index_simulation);
%! nb_line_data = size(lai_value,2);
%! classes_value = repmat({'"NaN"'},nb_line_data,1);
%! classes_value(1) = '"classe ginette"';
%! msdnom_value = repmat({'"NaN"'},nb_line_data,1);
%! msdnom_value(1) = '"msdnom ginette"';
%! zi_code='simul';
%! yyyy=uint16(2008);
%! result = write_lai_gis(lon_value, lat_value, datejul_value, datestr_value, lai_value, classes_value, msdnom_value, zi_code, yyyy);

%!demo
%! % From real file :file:`${PROJECT_ID}/LAI_CVS/LAI2008F_veg2010_sol_barkeji.csv
%! % to write :file:`${PROJECT_ID}/LAI_CSV/LAI2008F_veg2010_sol_barkedji_write_lai_gis.csv`
%! clear all
%! close all
%! varamma_startup
%! more off
%! yyyy = uint16(2008);
%! nbligne = uint16(0);
%! zi_code = 'barkedji';
%! [lon_value, lat_value, datejul_value, datestr_value, lai_value, classes_value, msdnom_value] = read_lai_gis(yyyy, nbligne, zi_code);
%! whos lat_value
%! result = write_lai_gis(lon_value, lat_value, datejul_value, datestr_value, lai_value, classes_value, msdnom_value, zi_code, yyyy);
